@@ -31,7 +31,7 @@ static bool set_auth(ConfElem *elem, const char *val, PgSocket *console);
 static const char *get_auth(ConfElem *elem);
 static bool set_defer_accept(ConfElem *elem, const char *val, PgSocket *console);
 
-static const char *usage_str =
+static const char usage_str[] =
 "Usage: %s [OPTION]... config.ini\n"
 "  -d            Run in background (as a daemon)\n"
 "  -R            Do a online restart\n"
@@ -64,6 +64,7 @@ char *cf_config_file = "";
 
 char *cf_listen_addr = NULL;
 int cf_listen_port = 6432;
+int cf_listen_backlog = 128;
 #ifndef WIN32
 char *cf_unix_socket_dir = "/tmp";
 #else
@@ -110,6 +111,7 @@ usec_t cf_server_idle_timeout = 10*60*USEC;
 usec_t cf_server_connect_timeout = 15*USEC;
 usec_t cf_server_login_retry = 15*USEC;
 usec_t cf_query_timeout = 0*USEC;
+usec_t cf_query_wait_timeout = 0*USEC;
 usec_t cf_client_idle_timeout = 0*USEC;
 usec_t cf_client_login_timeout = 60*USEC;
 usec_t cf_suspend_timeout = 10*USEC;
@@ -141,6 +143,7 @@ ConfElem bouncer_params[] = {
 {"pidfile",		false, CF_STR, &cf_pidfile},
 {"listen_addr",		false, CF_STR, &cf_listen_addr},
 {"listen_port",		false, CF_INT, &cf_listen_port},
+{"listen_backlog",	false, CF_INT, &cf_listen_backlog},
 #ifndef WIN32
 {"unix_socket_dir",	false, CF_STR, &cf_unix_socket_dir},
 #endif
@@ -163,6 +166,7 @@ ConfElem bouncer_params[] = {
 {"server_check_query",	true, CF_STR, &cf_server_check_query},
 {"server_check_delay",	true, CF_TIME, &cf_server_check_delay},
 {"query_timeout",	true, CF_TIME, &cf_query_timeout},
+{"query_wait_timeout",	true, CF_TIME, &cf_query_wait_timeout},
 {"client_idle_timeout",	true, CF_TIME, &cf_client_idle_timeout},
 {"client_login_timeout",true, CF_TIME, &cf_client_login_timeout},
 {"server_lifetime",	true, CF_TIME, &cf_server_lifetime},
@@ -720,7 +724,8 @@ int main(int argc, char *argv[])
 
 	/* initialize subsystems, order important */
 	srandom(time(NULL) ^ getpid());
-	event_init();
+	if (!event_init())
+		fatal("event_init() failed");
 	signal_setup();
 	janitor_setup();
 	stats_setup();
