@@ -509,6 +509,28 @@ bool find_server(PgSocket *client)
 			else
 				break;
 		}
+
+		/*
+		 * Don't let clients queue at all, if there is no working server connection.
+		 *
+		 * It must still allow following cases:
+		 * - empty pool on startup
+		 * - idle pool where all servers are removed
+		 *
+		 * Current logic:
+		 * - old server connections will be dropped by query_timeout
+		 * - new server connections fail due to server_connect_timeout, or other failure
+		 */
+		if (!server && pool->last_connect_failed) {
+			int cnt = pool_server_count(pool) - statlist_count(&pool->new_server_list);
+			if (!cnt) {
+				/* usual relaunch wont work, as there are no waiting clients */
+				launch_new_connection(client->pool);
+
+				disconnect_client(client, true, "no working server connection");
+				return false;
+			}
+		}
 	}
 	Assert(!server || server->state == SV_IDLE);
 
@@ -626,7 +648,7 @@ bool release_server(PgSocket *server)
 		pool->last_connect_failed = 0;
 		break;
 	default:
-		fatal("bad server state in release_server");
+		fatal("bad server state in release_server (%d)", server->state);
 	}
 
 	/* enforce lifetime immidiately on release */
@@ -655,13 +677,20 @@ bool release_server(PgSocket *server)
 }
 
 /* drop server connection */
-void disconnect_server(PgSocket *server, bool notify, const char *reason)
+void disconnect_server(PgSocket *server, bool notify, const char *reason, ...)
 {
 	PgPool *pool = server->pool;
 	PgSocket *client = server->link;
 	static const uint8_t pkt_term[] = {'X', 0,0,0,4};
 	int send_term = 1;
 	usec_t now = get_cached_time();
+	char buf[128];
+	va_list ap;
+
+	va_start(ap, reason);
+	vsnprintf(buf, sizeof(buf), reason, ap);
+	va_end(ap);
+	reason = buf;
 
 	if (cf_log_disconnections)
 		slog_info(server, "closing because: %s (age=%llu)", reason,
@@ -673,7 +702,7 @@ void disconnect_server(PgSocket *server, bool notify, const char *reason)
 		if (client) {
 			client->link = NULL;
 			server->link = NULL;
-			disconnect_client(client, true, reason);
+			disconnect_client(client, true, "%s", reason);
 		}
 		break;
 	case SV_TESTED:
@@ -691,7 +720,7 @@ void disconnect_server(PgSocket *server, bool notify, const char *reason)
 			send_term = 0;
 		break;
 	default:
-		fatal("disconnect_server: bad server state");
+		fatal("disconnect_server: bad server state (%d)", server->state);
 	}
 
 	Assert(server->link == NULL);
@@ -709,10 +738,17 @@ void disconnect_server(PgSocket *server, bool notify, const char *reason)
 }
 
 /* drop client connection */
-void disconnect_client(PgSocket *client, bool notify, const char *reason)
+void disconnect_client(PgSocket *client, bool notify, const char *reason, ...)
 {
+	char buf[128];
+	va_list ap;
 	usec_t now = get_cached_time();
 
+	va_start(ap, reason);
+	vsnprintf(buf, sizeof(buf), reason, ap);
+	va_end(ap);
+	reason = buf;
+
 	if (cf_log_disconnections)
 		slog_info(client, "closing because: %s (age=%llu)", reason,
 			  (now - client->connect_time) / USEC);
@@ -931,8 +967,7 @@ found:
 		/* notify readiness */
 		SEND_ReadyForQuery(res, main_client);
 		if (!res)
-			disconnect_client(main_client, true,
-					  "ReadyForQuery for main_client failed");
+			disconnect_client(main_client, true, "ReadyForQuery for main_client failed");
 		return;
 	}
 
